@@ -29,7 +29,6 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #ifdef G_OS_WIN32
-#  include <windows.h>
 #  include <winsock2.h>
 #  include <io.h>
 #  define close _close
@@ -48,7 +47,7 @@
 #include "../gcutter/gcut-assertions-helper.h"
 
 #ifndef CUT_DISABLE_SOCKET_SUPPORT
-#  ifdef HAVE_WINSOCK2_H
+#  ifdef CUT_HAVE_WINSOCK2_H
 #    include <winsock2.h>
 #    include <ws2tcpip.h>
 #  else
@@ -221,6 +220,31 @@ cut_utils_equal_string_array (gchar **strings1, gchar **strings2)
     return TRUE;
 }
 
+static void
+cut_utils_inspect_string_to_gstring (GString *inspected, const gchar *string)
+{
+    if (!string) {
+        g_string_append(inspected, "(null)");
+        return;
+    }
+
+    g_string_append_c(inspected, '"');
+    for (; string[0]; string++) {
+        switch (string[0]) {
+        case '"':
+            g_string_append(inspected, "\\\"");
+            break;
+        case '\\':
+            g_string_append(inspected, "\\\\");
+            break;
+        default:
+            g_string_append_c(inspected, string[0]);
+            break;
+        }
+    }
+    g_string_append_c(inspected, '"');
+}
+
 gchar *
 cut_utils_inspect_string_array (gchar **strings)
 {
@@ -233,7 +257,7 @@ cut_utils_inspect_string_array (gchar **strings)
     inspected = g_string_new("[");
     string = strings;
     while (*string) {
-        g_string_append_printf(inspected, "\"%s\"", *string);
+        cut_utils_inspect_string_to_gstring(inspected, *string);
         next_string = string + 1;
         if (*next_string)
             g_string_append(inspected, ", ");
@@ -244,13 +268,14 @@ cut_utils_inspect_string_array (gchar **strings)
     return g_string_free(inspected, FALSE);
 }
 
-const gchar *
+gchar *
 cut_utils_inspect_string (const gchar *string)
 {
-    if (string)
-        return string;
-    else
-        return "(null)";
+    GString *inspected;
+
+    inspected = g_string_new(NULL);
+    cut_utils_inspect_string_to_gstring(inspected, string);
+    return g_string_free(inspected, FALSE);
 }
 
 #ifndef CUT_DISABLE_SOCKET_SUPPORT
@@ -317,6 +342,7 @@ cut_utils_equal_sockaddr (const struct sockaddr *address1,
 }
 
 #ifdef G_OS_WIN32
+#  if !defined(NTDDI_VERSION) || (NTDDI_VERSION < NTDDI_LONGHORN)
 static const char *
 inet_ntop (int address_family, const void *source,
            char *destination, socklen_t destination_length)
@@ -343,6 +369,7 @@ inet_ntop (int address_family, const void *source,
         return destination;
     }
 }
+#  endif
 #endif
 
 gchar *
@@ -441,7 +468,7 @@ cut_utils_get_fixture_data_string (CutTestContext *context,
     va_start(args, path);
     data = cut_utils_get_fixture_data_string_va_list(context, fixture_data_path,
                                                      path, args);
-    va_end(args); /* FIXME: should be freed */
+    va_end(args);
 
     return data;
 }
@@ -462,8 +489,6 @@ cut_utils_get_fixture_data_string_va_list (CutTestContext *context,
     if (error) {
         gchar *inspected, *message;
 
-        g_free(*fixture_data_path);
-
         inspected = gcut_error_inspect(error);
         message = g_strdup_printf("can't get fixture data: %s", inspected);
         g_error_free(error);
