@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -40,13 +40,11 @@ gcut_assert_equal_type_helper (GType           expected,
     if (expected == actual) {
         cut_test_pass();
     } else {
-        cut_test_fail(cut_take_printf("<%s == %s>\n"
-                                      "expected: <%s>\n"
-                                      "  actual: <%s>",
+        cut_set_expected(g_type_name(expected));
+        cut_set_actual(g_type_name(actual));
+        cut_test_fail(cut_take_printf("<%s == %s>",
                                       expression_expected,
-                                      expression_actual,
-                                      g_type_name(expected),
-                                      g_type_name(actual)));
+                                      expression_actual));
     }
 }
 
@@ -68,13 +66,13 @@ gcut_assert_equal_value_helper (GValue         *expected,
         expected_type_name = g_type_name(G_VALUE_TYPE(expected));
         actual_type_name = g_type_name(G_VALUE_TYPE(actual));
 
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s> (%s)\n"
-                                  "  actual: <%s> (%s)",
-                                  expression_expected, expression_actual,
-                                  inspected_expected, expected_type_name,
-                                  inspected_actual, actual_type_name);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
+        message = cut_take_printf("<%s == %s>",
+                                  expression_expected, expression_actual);
+        cut_set_expected(cut_take_printf("%s (%s)",
+                                         inspected_expected,
+                                         expected_type_name));
+        cut_set_actual(cut_take_printf("%s (%s)",
+                                       inspected_actual, actual_type_name));
         cut_test_fail(message);
     }
 }
@@ -92,7 +90,6 @@ gcut_assert_equal_list_helper (const GList    *expected,
     if (gcut_list_equal(expected, actual, equal_function)) {
         cut_test_pass();
     } else {
-        const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected =
@@ -104,15 +101,11 @@ gcut_assert_equal_list_helper (const GList    *expected,
                                               inspect_function,
                                               inspect_user_data));
 
-        message = cut_take_printf("<%s(%s[i], %s[i]) == TRUE>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_equal_function,
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
+        cut_test_fail(cut_take_printf("<%s(%s[i], %s[i]) == TRUE>",
+                                      expression_equal_function,
+                                      expression_expected, expression_actual));
     }
 }
 
@@ -125,20 +118,15 @@ gcut_assert_equal_list_int_helper (const GList    *expected,
     if (gcut_list_equal_int(expected, actual)) {
         cut_test_pass();
     } else {
-        const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected = cut_take_string(gcut_list_inspect_int(expected));
         inspected_actual = cut_take_string(gcut_list_inspect_int(actual));
 
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
+        cut_test_fail(cut_take_printf("<%s == %s>",
+                                      expression_expected, expression_actual));
     }
 }
 
@@ -151,20 +139,15 @@ gcut_assert_equal_list_uint_helper (const GList    *expected,
     if (gcut_list_equal_uint(expected, actual)) {
         cut_test_pass();
     } else {
-        const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected = cut_take_string(gcut_list_inspect_uint(expected));
         inspected_actual = cut_take_string(gcut_list_inspect_uint(actual));
 
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
+        cut_test_fail(cut_take_printf("<%s == %s>",
+                                      expression_expected, expression_actual));
     }
 }
 
@@ -183,13 +166,10 @@ gcut_assert_equal_list_string_helper (const GList    *expected,
         inspected_expected = cut_take_string(gcut_list_inspect_string(expected));
         inspected_actual = cut_take_string(gcut_list_inspect_string(actual));
 
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
+        message = cut_take_printf("<%s == %s>",
+                                  expression_expected, expression_actual);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
         cut_test_fail(message);
     }
 }
@@ -205,7 +185,6 @@ gcut_assert_equal_list_enum_helper (GType           type,
     if (gcut_list_equal_int(expected, actual)) {
         cut_test_pass();
     } else {
-        const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected =
@@ -213,14 +192,10 @@ gcut_assert_equal_list_enum_helper (GType           type,
         inspected_actual =
             cut_take_string(gcut_list_inspect_enum(type, actual));
 
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
+        cut_test_fail(cut_take_printf("<%s == %s>",
+                                      expression_expected, expression_actual));
     }
 }
 
@@ -235,7 +210,6 @@ gcut_assert_equal_list_flags_helper (GType           type,
     if (gcut_list_equal_uint(expected, actual)) {
         cut_test_pass();
     } else {
-        const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected =
@@ -243,14 +217,10 @@ gcut_assert_equal_list_flags_helper (GType           type,
         inspected_actual =
             cut_take_string(gcut_list_inspect_flags(type, actual));
 
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
+        cut_test_fail(cut_take_printf("<%s == %s>",
+                                      expression_expected, expression_actual));
     }
 }
 
@@ -264,7 +234,6 @@ gcut_assert_equal_list_object_helper (const GList    *expected,
     if (gcut_list_equal(expected, actual, equal_function)) {
         cut_test_pass();
     } else {
-        const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected =
@@ -272,14 +241,10 @@ gcut_assert_equal_list_object_helper (const GList    *expected,
         inspected_actual =
             cut_take_string(gcut_list_inspect_object(actual));
 
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
+        cut_test_fail(cut_take_printf("<%s == %s>",
+                                      expression_expected, expression_actual));
     }
 }
 
@@ -297,28 +262,34 @@ gcut_assert_equal_hash_table_helper (GHashTable  *expected,
     if (gcut_hash_table_equal(expected, actual, equal_function)) {
         cut_test_pass();
     } else {
-        const gchar *message;
         gchar *inspected_expected, *inspected_actual;
+        GCompareFunc compare_function = NULL;
 
-        inspected_expected = gcut_hash_table_inspect(expected,
-                                                     key_inspect_function,
-                                                     value_inspect_function,
-                                                     inspect_user_data);
-        inspected_actual = gcut_hash_table_inspect(actual,
-                                                   key_inspect_function,
-                                                   value_inspect_function,
-                                                   inspect_user_data);
-        message = cut_take_printf("<%s(%s[key], %s[key]) == TRUE>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_equal_function,
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
+        if (key_inspect_function == gcut_inspect_direct) {
+            compare_function = cut_utils_compare_direct;
+        } else if (key_inspect_function == gcut_inspect_string) {
+            compare_function = cut_utils_compare_string;
+        }
+        inspected_expected =
+            gcut_hash_table_inspect_sorted(expected,
+                                           key_inspect_function,
+                                           value_inspect_function,
+                                           compare_function,
+                                           inspect_user_data);
+        inspected_actual =
+            gcut_hash_table_inspect_sorted(actual,
+                                           key_inspect_function,
+                                           value_inspect_function,
+                                           compare_function,
+                                           inspect_user_data);
+
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
         g_free(inspected_expected);
         g_free(inspected_actual);
-        cut_test_fail(message);
+        cut_test_fail(cut_take_printf("<%s(%s[key], %s[key]) == TRUE>",
+                                      expression_equal_function,
+                                      expression_expected, expression_actual));
     }
 }
 
@@ -331,23 +302,20 @@ gcut_assert_equal_hash_table_string_string_helper (GHashTable  *expected,
     if (gcut_hash_table_string_equal(expected, actual)) {
         cut_test_pass();
     } else {
-        const gchar *message;
         gchar *inspected_expected, *inspected_actual;
 
         inspected_expected =
             gcut_hash_table_string_string_inspect(expected);
         inspected_actual =
             gcut_hash_table_string_string_inspect(actual);
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected, expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
+
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
         g_free(inspected_expected);
         g_free(inspected_actual);
-        cut_test_fail(message);
+
+        cut_test_fail(cut_take_printf("<%s == %s>",
+                                      expression_expected, expression_actual));
     }
 }
 
@@ -377,23 +345,11 @@ gcut_assert_equal_error_helper (const GError   *expected,
     if (gcut_error_equal(expected, actual)) {
         cut_test_pass();
     } else {
-        const gchar *inspected_expected;
-        const gchar *inspected_actual;
-        const gchar *message;
-
-        inspected_expected = cut_take_string(gcut_error_inspect(expected));
-        inspected_actual = cut_take_string(gcut_error_inspect(actual));
-        message = cut_take_printf("<%s == %s>\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected,
-                                  expression_actual,
-                                  inspected_expected,
-                                  inspected_actual);
-        if (expected && actual)
-            message = cut_append_diff(message,
-                                      inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(cut_take_string(gcut_error_inspect(expected)));
+        cut_set_actual(cut_take_string(gcut_error_inspect(actual)));
+        cut_test_fail(cut_take_printf("<%s == %s>",
+                                      expression_expected,
+                                      expression_actual));
     }
 }
 
@@ -445,22 +401,17 @@ gcut_assert_equal_enum_helper (GType           enum_type,
     } else {
         const gchar *inspected_expected;
         const gchar *inspected_actual;
-        const gchar *message;
 
         inspected_expected =
             cut_take_string(gcut_enum_inspect(enum_type, expected));
         inspected_actual =
             cut_take_string(gcut_enum_inspect(enum_type, actual));
-        message = cut_take_printf("<%s == %s> (%s)\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected,
-                                  expression_actual,
-                                  expression_enum_type,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
+        cut_test_fail(cut_take_printf("<%s == %s> (%s)",
+                                      expression_expected,
+                                      expression_actual,
+                                      expression_enum_type));
     }
 }
 
@@ -477,22 +428,17 @@ gcut_assert_equal_flags_helper (GType           flags_type,
     } else {
         const gchar *inspected_expected;
         const gchar *inspected_actual;
-        const gchar *message;
 
         inspected_expected =
             cut_take_string(gcut_flags_inspect(flags_type, expected));
         inspected_actual =
             cut_take_string(gcut_flags_inspect(flags_type, actual));
-        message = cut_take_printf("<%s == %s> (%s)\n"
-                                  "expected: <%s>\n"
-                                  "  actual: <%s>",
-                                  expression_expected,
-                                  expression_actual,
-                                  expression_flags_type,
-                                  inspected_expected,
-                                  inspected_actual);
-        message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail(message);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
+        cut_test_fail(cut_take_printf("<%s == %s> (%s)",
+                                      expression_expected,
+                                      expression_actual,
+                                      expression_flags_type));
     }
 }
 
@@ -508,38 +454,23 @@ gcut_assert_equal_object_helper (GObject        *expected,
         cut_test_pass();
     } else {
         gchar *inspected_expected, *inspected_actual;
-        GString *message;
-        const gchar *fail_message;
+        const gchar *message;
 
         inspected_expected = gcut_object_inspect(expected);
         inspected_actual = gcut_object_inspect(actual);
-        message = g_string_new(NULL);
-
-        if (expression_equal_function)
-            g_string_append_printf(message,
-                                   "<%s(%s, %s)>\n",
-                                   expression_equal_function,
-                                   expression_expected, expression_actual);
-        else
-            g_string_append_printf(message,
-                                   "<%s == %s>\n",
-                                   expression_expected, expression_actual);
-        g_string_append_printf(message,
-                               "expected: <%s>\n"
-                               "  actual: <%s>",
-                               inspected_expected,
-                               inspected_actual);
-        if (expected && actual)
-            fail_message = cut_append_diff(message->str,
-                                           inspected_expected,
-                                           inspected_actual);
-        else
-            fail_message = cut_take_strdup(message->str);
+        cut_set_expected(inspected_expected);
+        cut_set_actual(inspected_actual);
         g_free(inspected_expected);
         g_free(inspected_actual);
-        g_string_free(message, TRUE);
 
-        cut_test_fail(fail_message);
+        if (expression_equal_function)
+            message = cut_take_printf("<%s(%s, %s)>",
+                                      expression_equal_function,
+                                      expression_expected, expression_actual);
+        else
+            message = cut_take_printf("<%s == %s>",
+                                      expression_expected, expression_actual);
+        cut_test_fail(message);
     }
 }
 
@@ -552,12 +483,27 @@ gcut_assert_equal_int64_helper (gint64          expected,
     if (expected == actual) {
         cut_test_pass();
     } else {
-        cut_test_fail(cut_take_printf("<%s == %s>\n"
-                                      "expected: <%" G_GINT64_FORMAT ">\n"
-                                      "  actual: <%" G_GINT64_FORMAT ">",
+        cut_set_expected(cut_take_printf("%" G_GINT64_FORMAT, expected));
+        cut_set_actual(cut_take_printf("%" G_GINT64_FORMAT, actual));
+        cut_test_fail(cut_take_printf("<%s == %s>",
+                                      expression_expected, expression_actual));
+    }
+}
+
+void
+gcut_assert_not_equal_int64_helper (gint64         expected,
+                                    gint64         actual,
+                                    const char     *expression_expected,
+                                    const char     *expression_actual)
+{
+    if (expected != actual) {
+        cut_test_pass();
+    } else {
+        cut_set_expected(cut_take_printf("%" G_GINT64_FORMAT, expected));
+        cut_set_actual(cut_take_printf("%" G_GINT64_FORMAT, actual));
+        cut_test_fail(cut_take_printf("<%s != %s>",
                                       expression_expected,
-                                      expression_actual,
-                                      expected, actual));
+                                      expression_actual));
     }
 }
 
@@ -570,12 +516,28 @@ gcut_assert_equal_uint64_helper (guint64         expected,
     if (expected == actual) {
         cut_test_pass();
     } else {
-        cut_test_fail(cut_take_printf("<%s == %s>\n"
-                                      "expected: <%" G_GUINT64_FORMAT ">\n"
-                                      "  actual: <%" G_GUINT64_FORMAT ">",
+        cut_set_expected(cut_take_printf("%" G_GUINT64_FORMAT, expected));
+        cut_set_actual(cut_take_printf("%" G_GUINT64_FORMAT, actual));
+        cut_test_fail(cut_take_printf("<%s == %s>",
                                       expression_expected,
-                                      expression_actual,
-                                      expected, actual));
+                                      expression_actual));
+    }
+}
+
+void
+gcut_assert_not_equal_uint64_helper (guint64         expected,
+                                     guint64         actual,
+                                     const char     *expression_expected,
+                                     const char     *expression_actual)
+{
+    if (expected != actual) {
+        cut_test_pass();
+    } else {
+        cut_set_expected(cut_take_printf("%" G_GUINT64_FORMAT, expected));
+        cut_set_actual(cut_take_printf("%" G_GUINT64_FORMAT, actual));
+        cut_test_fail(cut_take_printf("<%s != %s>",
+                                      expression_expected,
+                                      expression_actual));
     }
 }
 
@@ -593,12 +555,11 @@ gcut_assert_equal_pid_helper (GPid            expected,
     if (expected == actual) {
         cut_test_pass();
     } else {
-        cut_test_fail(cut_take_printf("<%s == %s>\n"
-                                      "expected: <%" GCUT_GPID_FORMAT ">\n"
-                                      "  actual: <%" GCUT_GPID_FORMAT ">",
+        cut_set_expected(cut_take_printf("%" GCUT_GPID_FORMAT, expected));
+        cut_set_actual(cut_take_printf("%" GCUT_GPID_FORMAT, actual));
+        cut_test_fail(cut_take_printf("<%s == %s>",
                                       expression_expected,
-                                      expression_actual,
-                                      expected, actual));
+                                      expression_actual));
     }
 }
 
@@ -611,12 +572,11 @@ gcut_assert_not_equal_pid_helper (GPid            expected,
     if (expected != actual) {
         cut_test_pass();
     } else {
-        cut_test_fail(cut_take_printf("<%s != %s>\n"
-                                      "expected: <%" GCUT_GPID_FORMAT ">\n"
-                                      "  actual: <%" GCUT_GPID_FORMAT ">",
+        cut_set_expected(cut_take_printf("%" GCUT_GPID_FORMAT, expected));
+        cut_set_actual(cut_take_printf("%" GCUT_GPID_FORMAT, actual));
+        cut_test_fail(cut_take_printf("<%s != %s>",
                                       expression_expected,
-                                      expression_actual,
-                                      expected, actual));
+                                      expression_actual));
     }
 }
 
