@@ -102,6 +102,7 @@ void test_not_equal_size(void);
 void test_equal_char(void);
 void test_not_equal_char(void);
 void test_equal_string(void);
+void test_equal_string_with_null(void);
 void test_equal_string_with_diff(void);
 void test_equal_string_with_folded_diff(void);
 void test_not_equal_string(void);
@@ -139,6 +140,8 @@ void test_equal_fixture_data_string (void);
 void test_equal_fixture_data_string_without_file (void);
 void test_error_errno (void);
 void test_equal_sockaddr (void);
+void test_equal_file_raw (void);
+void test_not_equal_file_raw (void);
 
 static gboolean compare_function_is_called;
 static gchar *tmp_file_name;
@@ -1342,11 +1345,32 @@ test_equal_string (void)
     cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
                            "assert-equal-string", NULL,
                            "<\"abc\" == \"ABC\">",
-                           "abc", "ABC",
+                           "\"abc\"", "\"ABC\"",
                            FAIL_LOCATION, "stub_equal_string",
                            NULL);
 }
 
+static void
+stub_equal_string_with_null (void)
+{
+    MARK_FAIL(cut_assert_equal_string("abc", NULL));
+}
+
+void
+test_equal_string_with_null (void)
+{
+    test = cut_test_new("assert-equal-string-with-null",
+                        stub_equal_string_with_null);
+    cut_assert_false(run());
+    cut_assert_test_result_summary(run_context, 1, 0, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
+                           "assert-equal-string-with-null", NULL,
+                           "<\"abc\" == NULL>",
+                           "\"abc\"", "(null)",
+                           FAIL_LOCATION, "stub_equal_string_with_null",
+                           NULL);
+}
+
 static void
 stub_not_equal_string (void)
 {
@@ -1365,7 +1389,7 @@ test_not_equal_string (void)
     cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
                            "assert-not-equal-string", NULL,
                            "<\"abc\" != \"abc\">",
-                           "abc", "abc",
+                           "\"abc\"", "\"abc\"",
                            FAIL_LOCATION, "stub_not_equal_string",
                            NULL);
 }
@@ -1389,14 +1413,14 @@ test_equal_string_with_diff (void)
     cut_assert_equal_string("<\"abc def ghi jkl\" == \"abc DEF ghi jkl\">",
                             cut_test_result_get_system_message(test_result));
     cut_assert_equal_string("<\"abc def ghi jkl\" == \"abc DEF ghi jkl\">\n"
-                            "expected: <abc def ghi jkl>\n"
-                            "  actual: <abc DEF ghi jkl>\n"
+                            "expected: <\"abc def ghi jkl\">\n"
+                            "  actual: <\"abc DEF ghi jkl\">\n"
                             "\n"
                             "diff:\n"
-                            "- abc def ghi jkl\n"
-                            "?     ^^^\n"
-                            "+ abc DEF ghi jkl\n"
-                            "?     ^^^",
+                            "- \"abc def ghi jkl\"\n"
+                            "?      ^^^\n"
+                            "+ \"abc DEF ghi jkl\"\n"
+                            "?      ^^^",
                             cut_test_result_get_message(test_result));
 }
 
@@ -1477,7 +1501,7 @@ test_equal_string_with_folded_diff (void)
                             "\"7123456789\" "
                             "\"8123456789\""
                             ">\n"
-                            "expected: <"
+                            "expected: <\""
                             "0123456789"
                             "1123456789"
                             "2123456789"
@@ -1487,8 +1511,8 @@ test_equal_string_with_folded_diff (void)
                             "6123456789"
                             "7123456789"
                             "8123456789"
-                            ">\n"
-                            "  actual: <"
+                            "\">\n"
+                            "  actual: <\""
                             "0000000000"
                             "1123456789"
                             "2123456789"
@@ -1498,10 +1522,10 @@ test_equal_string_with_folded_diff (void)
                             "6123456789"
                             "7123456789"
                             "8123456789"
-                            ">\n"
+                            "\">\n"
                             "\n"
                             "diff:\n"
-                            "- 0123456789"
+                            "- \"0123456789"
                             "1123456789"
                             "2123456789"
                             "3123456789"
@@ -1509,9 +1533,9 @@ test_equal_string_with_folded_diff (void)
                             "5123456789"
                             "6123456789"
                             "7123456789"
-                            "8123456789\n"
-                            "?  ^^^^^^^^^\n"
-                            "+ 0000000000"
+                            "8123456789\"\n"
+                            "?   ^^^^^^^^^\n"
+                            "+ \"0000000000"
                             "1123456789"
                             "2123456789"
                             "3123456789"
@@ -1519,30 +1543,30 @@ test_equal_string_with_folded_diff (void)
                             "5123456789"
                             "6123456789"
                             "7123456789"
-                            "8123456789\n"
-                            "?  ^^^^^^^^^\n"
+                            "8123456789\"\n"
+                            "?   ^^^^^^^^^\n"
                             "\n"
                             "folded diff:\n"
-                            "- 0123456789"
+                            "- \"0123456789"
                             "1123456789"
                             "2123456789"
                             "3123456789"
                             "4123456789"
                             "5123456789"
                             "6123456789"
-                            "71234567\n"
-                            "?  ^^^^^^^^^\n"
-                            "+ 0000000000"
+                            "7123456\n"
+                            "?   ^^^^^^^^^\n"
+                            "+ \"0000000000"
                             "1123456789"
                             "2123456789"
                             "3123456789"
                             "4123456789"
                             "5123456789"
                             "6123456789"
-                            "71234567\n"
-                            "?  ^^^^^^^^^\n"
-                            "  89"
-                            "8123456789",
+                            "7123456\n"
+                            "?   ^^^^^^^^^\n"
+                            "  789"
+                            "8123456789\"",
                             cut_test_result_get_message(test_result));
 }
 
@@ -1573,8 +1597,8 @@ test_equal_substring (void)
                            "<\"0123456789\" == "
                            "(actual_string + strlen(\"0000\") + 1)"
                            "[0..strlen(\"0123456789\")]>",
-                           "0123456789",
-                           "1234567899",
+                           "\"0123456789\"",
+                           "\"1234567899\"",
                            FAIL_LOCATION, "stub_equal_substring",
                            NULL);
 }
@@ -1608,8 +1632,8 @@ test_not_equal_substring (void)
                            "<\"0123456789\" != "
                            "(actual_string + strlen(\"0000\"))"
                            "[0..strlen(\"0123456789\")]>",
-                           "0123456789",
-                           "0123456789",
+                           "\"0123456789\"",
+                           "\"0123456789\"",
                            FAIL_LOCATION, "stub_not_equal_substring",
                            NULL);
 }
@@ -2064,7 +2088,7 @@ test_assert_message_with_format_string (void)
                            "stub-assert-message-with-string",
                            "expected and actual have format string",
                            "<\"%s\" == \"%d\">",
-                           "%s", "%d",
+                           "\"%s\"", "\"%d\"",
                            FAIL_LOCATION,
                            "stub_assert_message_with_format_string",
                            NULL);
@@ -2319,22 +2343,23 @@ error_errno (void)
     cut_error_errno(cut_message("Not error"));
 
     errno = EACCES;
-    cut_error_errno(cut_message("Should error"));
+    MARK_FAIL(cut_error_errno(cut_message("Should error")));
 }
 
 void
 test_error_errno (void)
 {
-    /* FIXME: use cut_assert_test_result_summary() */
-    CutTestResult *result;
-
     test = cut_test_new("error-errno", error_errno);
     cut_assert_false(run());
     cut_assert_test_result_summary(run_context, 1, 0, 0, 0, 1, 0, 0, 0);
-
-    result = cut_run_context_get_results(run_context)->data;
-    cut_assert_equal_string("Should error",
-                            cut_test_result_get_user_message(result));
+    cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_ERROR,
+                           "error-errno",
+                           "Should error",
+                           cut_take_printf("<%d> (%s)",
+                                           EACCES, g_strerror(EACCES)),
+                           NULL, NULL,
+                           FAIL_LOCATION, "error_errno",
+                           NULL);
 }
 
 #ifndef CUT_DISABLE_SOCKET_SUPPORT
@@ -2390,6 +2415,102 @@ test_equal_sockaddr (void)
 #endif
 }
 
+static void
+equal_file_raw (void)
+{
+    const gchar *data, *sub_data;
+
+    data = cut_build_path(cuttest_get_base_dir(),
+                          "fixtures", "assertions", "data.txt", NULL);
+    sub_data = cut_build_path(cuttest_get_base_dir(),
+                              "fixtures", "assertions", "sub", "data.txt", NULL);
+    cut_assert_equal_file_raw(data, data);
+    MARK_FAIL(cut_assert_equal_file_raw(data, sub_data));
+}
+
+void
+test_equal_file_raw (void)
+{
+    const gchar *data, *sub_data;
+
+    data = cut_build_path(cuttest_get_base_dir(),
+                          "fixtures", "assertions", "data.txt", NULL);
+    sub_data = cut_build_path(cuttest_get_base_dir(),
+                              "fixtures", "assertions", "sub", "data.txt", NULL);
+
+    test = cut_test_new("equal-file-raw", equal_file_raw);
+
+    cut_assert_false(run());
+    cut_assert_test_result_summary(run_context, 1, 5, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
+                           "equal-file-raw", NULL,
+                           "<content(data) == content(sub_data)>",
+                           cut_take_printf("path: <%s>\n"
+                                           "0x74 0x6f 0x70 0x20 "
+                                           "0x6c 0x65 0x76 0x65 0x6c 0x20 "
+                                           "0x64 0x61 0x74 0x61 0x0a: "
+                                           "top level data. "
+                                           "(size: 15)",
+                                           data),
+                           cut_take_printf("path: <%s>\n"
+                                           "0x73 0x75 0x62 0x20 "
+                                           "0x6c 0x65 0x76 0x65 0x6c 0x20 "
+                                           "0x64 0x61 0x74 0x61 0x0a: "
+                                           "sub level data. "
+                                           "(size: 15)",
+                                           sub_data),
+                           FAIL_LOCATION, "equal_file_raw",
+                           NULL);
+}
+
+static void
+not_equal_file_raw (void)
+{
+    const gchar *data, *sub_data;
+
+    data = cut_build_path(cuttest_get_base_dir(),
+                              "fixtures", "assertions", "data.txt", NULL);
+    sub_data = cut_build_path(cuttest_get_base_dir(),
+                              "fixtures", "assertions", "sub", "data.txt", NULL);
+    cut_assert_not_equal_file_raw(data, sub_data);
+    MARK_FAIL(cut_assert_not_equal_file_raw(data, data));
+}
+
+void
+test_not_equal_file_raw (void)
+{
+    const gchar *data, *sub_data;
+
+    data = cut_build_path(cuttest_get_base_dir(),
+                          "fixtures", "assertions", "data.txt", NULL);
+    sub_data = cut_build_path(cuttest_get_base_dir(),
+                              "fixtures", "assertions", "sub", "data.txt", NULL);
+
+    test = cut_test_new("not-equal-file-raw", not_equal_file_raw);
+
+    cut_assert_false(run());
+    cut_assert_test_result_summary(run_context, 1, 5, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
+                           "not-equal-file-raw", NULL,
+                           "<content(data) != content(data)>",
+                           cut_take_printf("path: <%s>\n"
+                                           "0x74 0x6f 0x70 0x20 "
+                                           "0x6c 0x65 0x76 0x65 0x6c 0x20 "
+                                           "0x64 0x61 0x74 0x61 0x0a: "
+                                           "top level data. "
+                                           "(size: 15)",
+                                           data),
+                           cut_take_printf("path: <%s>\n"
+                                           "0x74 0x6f 0x70 0x20 "
+                                           "0x6c 0x65 0x76 0x65 0x6c 0x20 "
+                                           "0x64 0x61 0x74 0x61 0x0a: "
+                                           "top level data. "
+                                           "(size: 15)",
+                                           data),
+                           FAIL_LOCATION, "not_equal_file_raw",
+                           NULL);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
