@@ -29,7 +29,7 @@
 #include "../lib/cuttest-utils.h"
 
 #ifndef CUT_DISABLE_SOCKET_SUPPORT
-#  ifdef HAVE_WINSOCK2_H
+#  ifdef G_OS_WIN32
 #    include <winsock2.h>
 #    include <ws2tcpip.h>
 #  else
@@ -46,6 +46,7 @@ void test_inspect_memory_with_printable (void);
 void test_inspect_memory_huge_data (void);
 void test_compare_string_array (void);
 void test_inspect_string_array (void);
+void test_inspect_string (void);
 void test_strv_concat (void);
 void test_remove_path_recursive (void);
 void test_fold (void);
@@ -176,6 +177,19 @@ test_inspect_string_array (void)
                                       cut_utils_inspect_string_array(strings));
 }
 
+void
+test_inspect_string (void)
+{
+    cut_assert_equal_string_with_free("(null)",
+                                      cut_utils_inspect_string(NULL));
+    cut_assert_equal_string_with_free("\"a\"",
+                                      cut_utils_inspect_string("a"));
+    cut_assert_equal_string_with_free("\"a\\\"b\"",
+                                      cut_utils_inspect_string("a\"b"));
+    cut_assert_equal_string_with_free("\"a\\\\b\"",
+                                      cut_utils_inspect_string("a\\b"));
+}
+
 void
 test_strv_concat (void)
 {
@@ -403,6 +417,7 @@ test_equal_sockaddr_in (void)
 #ifndef CUT_DISABLE_SOCKET_SUPPORT
 
 #  ifdef G_OS_WIN32
+#    if !defined(NTDDI_VERSION) || (NTDDI_VERSION < NTDDI_LONGHORN)
 static int
 inet_pton(int address_family, const char *source, void *destination)
 {
@@ -423,6 +438,7 @@ inet_pton(int address_family, const char *source, void *destination)
     return WSAStringToAddress((LPTSTR)source, address_family, NULL,
                               destination, &destination_length);
 }
+#    endif
 #  endif
 
 static void
